'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = getBrowserStackOptions;
function getBrowserStackOptions(config) {
  const bsConfig = config.bsConfig;
  const local = config.local;
  const sources = config.sources;
  const environment = config.environment;
  var _config$pkg = config.pkg;
  const pkg = _config$pkg === undefined ? {} : _config$pkg;
  const BROWSERSTACK_USERNAME = bsConfig.BROWSERSTACK_USERNAME;
  const BROWSERSTACK_API = bsConfig.BROWSERSTACK_API;
  const localIdentifier = bsConfig.localIdentifier;
  const devHost = sources.devHost;
  const devPort = sources.devPort;
  const hotPort = sources.hotPort;
  const branch = environment.branch;
  const name = pkg.name;
  const version = pkg.version;


  return {
    groupOptions: {
      project: `v${ version } [${ branch || 'local' }:e2e${ local ? ':debug' : '' }]`,
      build: name
    },
    authOptions: {
      user: BROWSERSTACK_USERNAME,
      key: BROWSERSTACK_API,
      host: 'hub.browserstack.com',
      port: 80
    },
    envOptions: {
      tunnel: {
        'browserstack.local': 'true',
        'browserstack.debug': 'true',
        'browserstack.localIdentifier': localIdentifier
      },
      ci: {
        'browserstack.debug': 'true',
        'browserstack.local': 'true',

        'browserstack.localIdentifier': localIdentifier
      }
    },
    spawnTunnelOptions: {
      key: BROWSERSTACK_API,
      hosts: [{
        name: devHost,
        port: devPort,
        sslFlag: 0
      }, {
        name: devHost,
        port: hotPort,
        sslFlag: 0
      }],
      v: true,
      localIdentifier,
      forcelocal: true
    }
  };
}
module.exports = exports['default'];