'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

exports.default = getCapabilities;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _boilerUtils = require('boiler-utils');

var _boilerUtils2 = _interopRequireDefault(_boilerUtils);

var _browsers = require('./browser-stack/browsers');

var _browsers2 = _interopRequireDefault(_browsers);

var _devices = require('./browser-stack/devices');

var _devices2 = _interopRequireDefault(_devices);

var _getBrowserStackOptions = require('./browser-stack/get-browser-stack-options');

var _getBrowserStackOptions2 = _interopRequireDefault(_getBrowserStackOptions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const gulpTaskUtils = _boilerUtils2.default.gulpTaskUtils;
const dynamicRequire = _boilerUtils2.default.dynamicRequire;
const logError = gulpTaskUtils.logError;
function getCapabilities(config) {
  let runnerOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  let forceTunnel = arguments[2];
  const environment = config.environment;
  const sources = config.sources;
  const branch = environment.branch;
  const isDevRoot = environment.isDevRoot;
  const isMaster = environment.isMaster;
  const devPort = sources.devPort;
  const devUrl = sources.devUrl;
  const internalHost = sources.internalHost;
  const devUrlOverride = runnerOptions.devUrl;

  function getDevUrl() {
    if (typeof devUrlOverride === 'function') {
      return devUrlOverride(config);
    }

    let baseUrl;

    const protocol = branch ? 'https://' : 'http://';
    if (branch) {
      const base = devUrlOverride ? devUrlOverride : `${ protocol }${ devUrl }`;

      if (isDevRoot || isMaster) {
        baseUrl = base;
      } else {
        baseUrl = `${ base }/${ branch }`;
      }
    } else {
      baseUrl = `${ protocol }${ internalHost }:${ devPort }`;
    }

    return baseUrl;
  }

  var _getBrowserStackOptio = (0, _getBrowserStackOptions2.default)(config);

  const groupOptions = _getBrowserStackOptio.groupOptions;
  const envOptions = _getBrowserStackOptio.envOptions;
  const authOptions = _getBrowserStackOptio.authOptions;


  let map = _immutable2.default.Map();
  let isLocal, testEnv;

  try {
    const makeTestFiles = dynamicRequire(require('./get-test-files'));
    map = makeTestFiles(config, runnerOptions);
  } catch (err) {
    logError({ err, plugin: '[selenium: get-capabilities]' });
  }

  const baseConfig = {
    baseUrl: getDevUrl(),
    logLevel: 'silent'
  };

  if (_lodash2.default.isUndefined(branch)) {
    const localBrowsers = ['chrome', 'firefox'];

    isLocal = true;

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = map.keys()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        const devices = _step.value;

        const tunnelDevices = devices.filter(device => localBrowsers.indexOf(device) === -1);


        if (tunnelDevices.size > 0 || forceTunnel) {
          isLocal = false;
          break;
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    testEnv = isLocal ? 'local' : 'tunnel';
  } else {
    testEnv = 'ci';
  }

  const allBsDevices = [].concat((0, _toConsumableArray3.default)(_browsers2.default), (0, _toConsumableArray3.default)(_devices2.default));
  const testConfig = [];

  if (!isLocal) {
    _lodash2.default.assign(baseConfig, authOptions);
  }

  map.forEach((fps, devices) => {
    const config = envOptions[testEnv];
    const specs = fps.toJS();

    let capabilities = _immutable2.default.List();
    devices.forEach(device => {
      const addToCapabilities = caps => {
        if (!isLocal) {
          const browserName = caps.browserName;
          const device = caps.device;

          _lodash2.default.assign(caps, groupOptions, { name: device || browserName });
        }

        capabilities = capabilities.push(_lodash2.default.assign({}, caps, config, isLocal));
      };

      if (_immutable2.default.Map.isMap(device)) {
        addToCapabilities(device.toJS());
      } else if (_lodash2.default.isString(device)) {
        const deviceArr = allBsDevices.filter(bsDevice => {
          const browserName = bsDevice.browserName;
          const configDevice = bsDevice.device;

          const re = new RegExp(device, 'i');

          return re.test(browserName) || re.test(configDevice);
        });

        if (deviceArr.length < 1) {
          logError({ err: new Error(`${ device } is not listed in default devices`), plugin: '[selenium: add-capabilities]' });
        }

        deviceArr.forEach(device => {
          addToCapabilities(isLocal ? { browserName: device.browserName } : device);
        });
      }
    });

    testConfig.push(_lodash2.default.assign({}, baseConfig, {
      specs,
      capabilities: _lodash2.default.isFunction(capabilities.toJS) ? capabilities.toJS() : capabilities
    }));
  });

  return {
    testEnv,
    testConfig
  };
}
module.exports = exports['default'];