'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _extends2 = require('babel-runtime/helpers/extends');

var _extends3 = _interopRequireDefault(_extends2);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

exports.makeSpecsGlob = makeSpecsGlob;
exports.default = getTestFiles;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _globby = require('globby');

var _globby2 = _interopRequireDefault(_globby);

var _immutable = require('immutable');

var _immutable2 = _interopRequireDefault(_immutable);

var _hackRequire = require('./hack-require');

var _hackRequire2 = _interopRequireDefault(_hackRequire);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function makeSpecsGlob(_ref) {
  let sources = _ref.sources;
  let utils = _ref.utils;
  let file = _ref.file;
  let mobile = _ref.mobile;
  let desktop = _ref.desktop;
  let runnerOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

  const specGlob = '*-spec';
  const testDir = sources.testDir;
  const specsDir = runnerOptions.specsDir;
  const addbase = utils.addbase;

  const specsBase = specsDir ? specsDir : (0, _path.join)(testDir, 'e2e');
  const hasMobile = _fs2.default.existsSync(addbase(specsBase, 'mobile')) && mobile;
  const hasDesktop = _fs2.default.existsSync(addbase(specsBase, 'desktop')) && desktop;
  let glob;

  if (hasMobile && hasDesktop) {
    glob = (0, _path.join)(specsBase, `**/${ file || specGlob }.js`);
  } else if (hasDesktop) {
    glob = (0, _path.join)(specsBase, `{desktop/**/,,!(mobile)/**/}${ file || specGlob }.js`);
  } else if (hasMobile) {
    glob = (0, _path.join)(specsBase, `{mobile/**/,,!(desktop)/**/}${ file || specGlob }.js`);
  } else {
    glob = (0, _path.join)(specsBase, `**/${ file || specGlob }.js`);
  }

  return glob;
}

function getTestFiles(config) {
  let runnerOptions = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  const force = config.force;
  const desktop = config.desktop;
  const mobile = config.mobile;
  const utils = config.utils;
  const logError = utils.logError;


  const types = { desktop, mobile };
  const specHook = _hackRequire2.default.init(/-spec\.js/, 'specs', types);
  const fp = makeSpecsGlob(config, runnerOptions);
  const globs = _globby2.default.sync(fp, { cwd: process.cwd() });
  const filesLen = globs.length;

  if (filesLen === 0) {
    logError({
      err: new Error(`Spec file path not found: ${ (0, _path.basename)(fp) }`),
      plugin: '[selenium: get-test-files]'
    });
  }

  const testFiles = globs.reduce((acc, globFp) => {
    const exportedData = require(globFp);
    const keys = Object.keys(types);
    const typeKeys = keys.filter(key => !!types[key]);
    const hasType = typeKeys.length > 0;
    const typesToTest = hasType ? typeKeys : keys;
    let devices;

    function retrieveDevices(specFileData) {
      return typesToTest.reduce((list, key) => {
        const cliDataByKey = types[key];
        const deviceKey = key === 'desktop' ? 'browserName' : 'device';
        let dataByKey;

        if (_lodash2.default.isBoolean(cliDataByKey)) {
          dataByKey = cliDataByKey && specFileData[key] ? specFileData[key] : [];
        } else if (specFileData[key]) {
          dataByKey = _lodash2.default.intersection(specFileData[key], _lodash2.default.isArray(cliDataByKey) ? cliDataByKey : []);
        } else {
          dataByKey = [];
        }

        const data = dataByKey.filter(data => {
          let ret = false;

          if (_lodash2.default.isUndefined(cliDataByKey) || cliDataByKey === true) {
            ret = true;
          } else if (cliDataByKey) {
            let deviceName;

            if (_lodash2.default.isPlainObject(data)) {
              deviceName = data[deviceKey];
            } else if (_lodash2.default.isString(data)) {
              deviceName = data;
            } else {
              logError({
                err: new Error(`data exported from ${ globFp } must be of type String or Object`),
                plugin: '[selenium: get-test-files]'
              });
            }

            if (deviceName) {
              ret = cliDataByKey.filter(device => new RegExp(deviceName.toLowerCase()).test(cliDataByKey.join(' ')));
            }
          }

          return ret;
        });

        return [].concat((0, _toConsumableArray3.default)(list), (0, _toConsumableArray3.default)(data));
      }, []);
    }

    devices = retrieveDevices(exportedData);

    if (devices.length === 0 && hasType && force) {
      devices = retrieveDevices(types);
    }

    const devicesByFp = devices.length ? { [globFp]: devices } : {};

    return (0, _extends3.default)({}, acc, devicesByFp);
  }, {});

  if (Object.keys(testFiles).length === 0) {
    const fps = globs.map(fp => (0, _path.basename)(fp)).join(', ');
    logError({
      err: new Error(`No devices found for : ${ fps }. Try specifying the --force option`),
      plugin: '[selenium: get-test-files]'
    });
  }

  _hackRequire2.default.unmount(specHook);

  const deviceMap = Object.keys(testFiles).reduce((map, fp) => {
    const devices = _immutable2.default.fromJS(testFiles[fp]);
    const accumulator = new Map();

    if (map.size) {
      devices.forEach(device => {
        let prevKey;

        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = map.keys()[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            const key = _step.value;

            if (_immutable2.default.is(device, key)) {
              prevKey = key;
            }
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        if (prevKey) {
          const prevPath = map.get(prevKey);

          accumulator.set(prevKey, prevPath.push(fp));
        } else {
          accumulator.set(device, _immutable2.default.fromJS([fp]));
        }
      });
    } else {
      devices.forEach(device => {
        accumulator.set(device, _immutable2.default.fromJS([fp]));
      });
    }

    return new Map([].concat((0, _toConsumableArray3.default)(map), (0, _toConsumableArray3.default)(accumulator)));
  }, new Map());

  const suite = new Map();

  deviceMap.forEach((fps, device) => {
    let remove;

    suite.forEach((savedFps, deviceList) => {
      if (_immutable2.default.is(fps, savedFps)) {
        remove = deviceList;
      }
    });

    if (suite.size === 0 || _lodash2.default.isUndefined(remove)) {
      suite.set(_immutable2.default.fromJS([device]), fps);
    } else if (remove) {
      suite.delete(remove);
      suite.set(remove.push(device), fps);
    }
  });

  return suite;
}