'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _requireHacker = require('require-hacker');

var _requireHacker2 = _interopRequireDefault(_requireHacker);

var _browsers = require('./browser-stack/browsers');

var _devices = require('./browser-stack/devices.js');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

exports.default = {
  init(pathRe, name, opts) {
    const babel = require('babel-core');
    const desktop = opts.desktop;
    const mobile = opts.mobile;

    const desktopRe = /^.+?\/desktop\/.+?\.jsx?$/;
    const mobileRe = /^.+?\/mobile\/.+?\.jsx?$/;
    const desktopCliArg = !_lodash2.default.isUndefined(desktop);
    const mobileCliArg = !_lodash2.default.isUndefined(mobile);

    return _requireHacker2.default.global_hook('specs', fp => {
      const isDesktop = desktopRe.test(fp);
      const isMobile = mobileRe.test(fp);
      let ret;

      if (pathRe.test(fp)) {
        const content = _fs2.default.readFileSync(fp, { encoding: 'utf8' });
        const re = /export default *\{/;
        const match = re.exec(content);
        let str;

        if (_lodash2.default.isArray(match)) {
          str = match[0];
          const rest = content.split(str).splice(-1)[0];
          let open = 1;

          for (let i = 0; i < rest.length; i += 1) {
            const chr = rest[i];

            if (!open) {
              break;
            } else if (chr === '}') {
              open -= 1;
            } else if (chr === '{') {
              open += 1;
            }

            str += chr;
          }
        } else if (desktopCliArg && isDesktop) {
          const defaults = {
            desktop: _lodash2.default.isArray(desktop) ? desktop : _browsers.defaults
          };

          str = `export default ${ JSON.stringify(defaults) }`;
        } else if (mobileCliArg && isMobile) {
          const defaults = {
            mobile: _lodash2.default.isArray(mobile) ? mobile : _devices.defaults
          };

          str = `export default ${ JSON.stringify(defaults) }`;
        } else if (!isMobile && !isDesktop) {
          const defaults = {};

          if (desktopCliArg) {
            _lodash2.default.assign(defaults, {
              desktop: _lodash2.default.isArray(desktop) ? desktop : _browsers.defaults
            });
          }

          if (mobileCliArg) {
            _lodash2.default.assign(defaults, {
              mobile: _lodash2.default.isArray(mobile) ? mobile : _devices.defaults
            });
          }

          str = `export default ${ JSON.stringify(defaults) }`;
        }

        var _babel$transform = babel.transform(str, {
          presets: ['es2015'],
          plugins: ['add-module-exports']
        });

        const code = _babel$transform.code;


        ret = code;
      }

      return ret;
    });
  },

  unmount(inst) {
    inst.unmount();
    return true;
  }
};
module.exports = exports['default'];