'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

exports.default = function (gulp, plugins, config) {
  return gulpCb => {
    const desktop = config.desktop;
    const mobile = config.mobile;
    const configFile = config.configFile;
    const metaData = config.metaData;
    const utils = config.utils;
    const getTaskName = utils.getTaskName;


    function exit(code, cb) {
      if (typeof gulpCb === 'function') {
        gulpCb();
      }

      process.exit(code);
    }

    const tunnelKey = 'tunnel';
    const taskName = getTaskName(metaData);
    const forceTunnel = taskName === tunnelKey;
    const noDevices = _lodash2.default.isUndefined(desktop) && _lodash2.default.isUndefined(mobile);
    const tunnelOnly = forceTunnel && noDevices;

    if (noDevices && !tunnelOnly) {
      config.desktop = true;
      config.mobile = true;
    }

    const runnerOptions = (0, _getTestConfig2.default)(configFile);

    var _ref = tunnelOnly ? {} : (0, _getCapabilities2.default)(config, runnerOptions, forceTunnel);

    var _ref$testEnv = _ref.testEnv;
    const testEnv = _ref$testEnv === undefined ? tunnelKey : _ref$testEnv;
    var _ref$testConfig = _ref.testConfig;
    const testConfig = _ref$testConfig === undefined ? [] : _ref$testConfig;

    const browserStackOptions = (0, _getBrowserStackOptions2.default)(config);

    var _runParentFn = runParentFn(arguments, { data: testConfig });

    const data = _runParentFn.data;


    function startRunner(cb) {
      (0, _spawn2.default)(data, runnerOptions, config, cb);
    }

    if (testEnv === 'tunnel') {
      const BrowserStackTunnel = require('browserstacktunnel-wrapper');
      const browserStackTunnel = new BrowserStackTunnel(browserStackOptions.spawnTunnelOptions);

      browserStackTunnel.on('started', () => log(browserStackTunnel.stdoutData));

      runGen(function* () {
        let startTunnel = thunk(browserStackTunnel.start, browserStackTunnel);
        try {
          yield startTunnel();
        } catch (err) {
          logError({ err, plugin: '[tunnel start]' });
        }

        if (tunnelOnly) {
          log('Visit BrowserStack Live to QA: https://www.browserstack.com/start');
        } else {
          let cp = thunk(startRunner);
          let code = yield cp();
          let stopTunnel = thunk(browserStackTunnel.stop, browserStackTunnel);

          try {
            yield stopTunnel();
          } catch (err) {
            logError({ err, plugin: '[tunnel stop]' });
          }

          exit(code);
        }
      });
    } else if (testEnv === 'local') {
      const selenium = require('selenium-standalone');
      const install = require('./selenium-standalone/install');

      runGen(function* () {
        let seleniumInstall = thunk(install);
        try {
          yield seleniumInstall(_options2.default.installOptions);
        } catch (err) {
          logError({ err, plugin: '[selenium install]' });
        }

        let seleniumStart = thunk(selenium.start, selenium);
        let child;

        try {
          child = yield seleniumStart(_options2.default.startOptions);
        } catch (err) {
          logError({ err, plugin: `[selenium start]: ${ err.message } => pkill java` });
        }

        let cp = thunk(startRunner);
        let code = yield cp();

        try {
          child.kill();
        } catch (err) {
          logError({ err, plugin: '[selenium: local server kill]' });
        }
        exit(code);
      });
    } else if (testEnv === 'ci') {
      runGen(function* () {
        let cp = thunk(startRunner);
        let code = yield cp();

        exit(code);
      });
    } else {
      logError({ err: new Error('Your test environment was not defined'), plugin: '[selenium]' });
    }
  };
};

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _boilerUtils = require('boiler-utils');

var _boilerUtils2 = _interopRequireDefault(_boilerUtils);

var _getTestConfig = require('./get-test-config');

var _getTestConfig2 = _interopRequireDefault(_getTestConfig);

var _options = require('./selenium-standalone/options');

var _options2 = _interopRequireDefault(_options);

var _getBrowserStackOptions = require('./browser-stack/get-browser-stack-options');

var _getBrowserStackOptions2 = _interopRequireDefault(_getBrowserStackOptions);

var _getCapabilities = require('./get-capabilities');

var _getCapabilities2 = _interopRequireDefault(_getCapabilities);

var _spawn = require('./runner/spawn');

var _spawn2 = _interopRequireDefault(_spawn);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const buildLogger = _boilerUtils2.default.buildLogger;
const thunk = _boilerUtils2.default.thunk;
const runGen = _boilerUtils2.default.runGen;
const runParentFn = _boilerUtils2.default.runParentFn;
const gulpTaskUtils = _boilerUtils2.default.gulpTaskUtils;
const logError = gulpTaskUtils.logError;
const log = buildLogger.log;
module.exports = exports['default'];