'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _objectWithoutProperties2 = require('babel-runtime/helpers/objectWithoutProperties');

var _objectWithoutProperties3 = _interopRequireDefault(_objectWithoutProperties2);

exports.default = runNightwatch;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _merge = require('lodash/merge');

var _merge2 = _interopRequireDefault(_merge);

var _child_process = require('child_process');

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _boilerUtils = require('boiler-utils');

var _boilerUtils2 = _interopRequireDefault(_boilerUtils);

var _makeConfig = require('./make-config');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const thunk = _boilerUtils2.default.thunk;
function runNightwatch(_ref) {
  let opt = _ref.opt;
  let concurrent = _ref.concurrent;
  let config = _ref.config;
  let runnerOptions = _ref.runnerOptions;
  let tmpDir = _ref.tmpDir;
  const specsDir = runnerOptions.specsDir;
  const commandsDir = runnerOptions.commandsDir;
  var _runnerOptions$custom = runnerOptions.customSettings;
  const customSettings = _runnerOptions$custom === undefined ? {} : _runnerOptions$custom;

  _fsExtra2.default.mkdirsSync(tmpDir);

  const addSeleniumKeys = seleniumConfig => {
    return Object.keys(seleniumConfig).reduce((acc, key) => {
      const val = seleniumConfig[key];
      switch (key) {
        case 'user':
          if (val) acc.desiredCapabilities['browserstack.user'] = val;
          break;
        case 'key':
          if (val) acc.desiredCapabilities['browserstack.key'] = val;
          break;
        case 'host':
          acc['selenium_host'] = val || 'localhost';
          break;
        case 'port':
          acc['selenium_port'] = val || 4444;
          break;
        case 'logLevel':
          if (val === 'silent') acc[val] = true;
          break;
      }

      return acc;
    }, {
      desiredCapabilities: {}
    });
  };

  const addCaps = seleniumConfig => {
    const baseUrl = seleniumConfig.baseUrl;
    const specs = seleniumConfig.specs;
    const capabilities = seleniumConfig.capabilities;
    const selenium = (0, _objectWithoutProperties3.default)(seleniumConfig, ['baseUrl', 'specs', 'capabilities']);


    return capabilities.reduce((acc, cap) => {
      const browserName = cap.browserName;

      const desiredCapabilities = Object.assign({}, cap);

      acc[browserName] = (0, _merge2.default)({}, _makeConfig.nightwatchDefaults, {
        launch_url: baseUrl,
        desiredCapabilities
      }, addSeleniumKeys(selenium));

      return acc;
    }, {});
  };

  const capabilities = opt.capabilities;
  const specs = opt.specs;

  const testSettings = addCaps(opt);
  const testTargets = capabilities.map((_ref2) => {
    let browserName = _ref2.browserName;
    return browserName;
  });
  const fps = specs.map(fp => _path2.default.basename(_path2.default.relative(specsDir, fp), _path2.default.extname(fp)));

  const emitter = new _events2.default();
  function onPromisesDone(code) {
    emitter.emit('close', code);
  }

  const promises = [];
  testTargets.forEach(target => {
    const configFp = _path2.default.resolve(tmpDir, `nightwatch-${ target }-${ new Date().getTime() }.json`);
    const json = {
      src_folders: [specsDir],
      custom_commands_path: commandsDir,
      output_folder: false,
      test_settings: { [target]: Object.assign({}, testSettings[target], customSettings) }
    };
    _fsExtra2.default.writeJsonSync(configFp, json);

    const args = {
      config: configFp,
      env: target
    };

    if (fps && fps.length) {
      Object.assign(args, { filter: fps.length > 1 ? `{${ fps.join(',') }}.js` : `${ fps[0] }.js` });
    }

    const promise = new Promise((res, rej) => {
      const cp = (0, _child_process.spawn)('node', [_path2.default.resolve(__dirname, 'runner.js'), JSON.stringify(args)], {
        stdio: 'inherit'
      });
      cp.on('close', code => {
        code ? rej(code) : res(code);
      });
    });
    promises.push(promise);
  });

  Promise.all(promises).then(() => onPromisesDone(0)).catch(code => onPromisesDone(code));

  return thunk(emitter.on, emitter);
}
module.exports = exports['default'];