'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = spawn;

var _fsExtra = require('fs-extra');

var _fsExtra2 = _interopRequireDefault(_fsExtra);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _boilerUtils = require('boiler-utils');

var _boilerUtils2 = _interopRequireDefault(_boilerUtils);

var _run = require('./webdriverio/run');

var _run2 = _interopRequireDefault(_run);

var _run3 = require('./nightwatch/run');

var _run4 = _interopRequireDefault(_run3);

var _parseBrowserNames = require('./parse-browser-names');

var _parseBrowserNames2 = _interopRequireDefault(_parseBrowserNames);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const tmpDir = _path2.default.join(__dirname, 'nightwatch', '.tmp');
const buildLogger = _boilerUtils2.default.buildLogger;
const runGen = _boilerUtils2.default.runGen;
const logError = _boilerUtils2.default.logError;
const log = buildLogger.log;
function spawn(testSettings, runnerOptions, config, cb) {
  const TRAVIS_BRANCH = process.env.TRAVIS_BRANCH;
  const runner = runnerOptions.runner;

  const isNightwatch = runner === 'nightwatch';

  runGen(function* () {
    const codes = [];
    const runCp = isNightwatch ? _run4.default : _run2.default;

    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
      for (var _iterator = testSettings[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
        const opt = _step.value;

        const cpOpts = {
          opt,
          runnerOptions,
          config,
          concurrent: !!TRAVIS_BRANCH
        };
        if (isNightwatch) {
          Object.assign(cpOpts, { tmpDir });
        }

        try {
          var _parseNames = (0, _parseBrowserNames2.default)(opt);

          const browsers = _parseNames.browsers;
          const specs = _parseNames.specs;

          log(`Child process for ${ browsers } testing ${ specs } starting`);
          const thunkedCp = runCp(cpOpts);
          const code = yield thunkedCp('close');
          codes.push(code);
          log(`Child process for ${ browsers } testing ${ specs } closed with status: ${ code }`);
        } catch (err) {
          logError({ err, plugin: 'selenium [spawn]' });
        }
      }
    } catch (err) {
      _didIteratorError = true;
      _iteratorError = err;
    } finally {
      try {
        if (!_iteratorNormalCompletion && _iterator.return) {
          _iterator.return();
        }
      } finally {
        if (_didIteratorError) {
          throw _iteratorError;
        }
      }
    }

    const errorCodes = codes.filter(code => code !== 0);
    if (errorCodes.length) {
      cb(errorCodes[0]);
    } else {
      cb(null, 0);
    }
  });

  if (isNightwatch) {
    process.on('exit', () => _fsExtra2.default.removeSync(tmpDir));
  }
}
module.exports = exports['default'];